@@ -26,8 +26,6 @@ Equipment Corporation.
 
 ******************************************************************/
 
-typedef int Bool;
-
 #ifdef HAVE_DIX_CONFIG_H
 #include <dix-config.h>
 #endif
@@ -41,12 +39,7 @@ Bool DPMSSupported(void)
     return FALSE;
 }
 
-int DPMSGet(int *plevel)
+int DPMSSet(ClientPtr client, int level)
 {
-    return -1;
-}
-
-void DPMSSet(int level)
-{
-
+    return Success;
 }
