@@ -51,6 +51,7 @@
 
 #include <X11/extensions/render.h>
 #include <X11/extensions/renderproto.h>
+#include <X11/extensions/Xfixes.h>
 #include "picture.h"
 #include "picturestr.h"
 #include "mipict.h"
@@ -144,7 +145,7 @@ Bool dmxPictureInit(ScreenPtr pScreen, PictFormatPtr formats, int nformats)
     if (!miPictureInit(pScreen, formats, nformats))
 	return FALSE;
 
-    if (!dixRequestPrivate(dmxPictPrivateKey, sizeof(dmxPictPrivRec)))
+    if (!dixRegisterPrivateKey(&dmxPictPrivateKeyRec, PRIVATE_PICTURE, sizeof(dmxPictPrivRec)))
 	return FALSE;
 
     ps = GetPictureScreen(pScreen);
@@ -273,7 +274,7 @@ static int dmxProcRenderCreateGlyphSet(ClientPtr client)
 	
 	glyphSet = SecurityLookupIDByType(client, stuff->gsid, GlyphSetType,
 					  DixDestroyAccess);
-	glyphPriv = xalloc(sizeof(dmxGlyphPrivRec));
+	glyphPriv = malloc(sizeof(dmxGlyphPrivRec));
 	if (!glyphPriv) return BadAlloc;
         glyphPriv->glyphSets = NULL;
         MAXSCREENSALLOC_RETURN(glyphPriv->glyphSets, BadAlloc);
@@ -330,7 +331,7 @@ static int dmxProcRenderFreeGlyphSet(ClientPtr client)
 	}
 
         MAXSCREENSFREE(glyphPriv->glyphSets);
-	xfree(glyphPriv);
+	free(glyphPriv);
 	DMX_SET_GLYPH_PRIV(glyphSet, NULL);
     }
 
@@ -368,7 +369,7 @@ static int dmxProcRenderAddGlyphs(ClientPtr client)
 		  sizeof(xRenderAddGlyphsReq) -
 		  (sizeof(CARD32) + sizeof(xGlyphInfo)) * nglyphs);
 
-        gidsCopy = xalloc(sizeof(*gidsCopy) * nglyphs);
+        gidsCopy = malloc(sizeof(*gidsCopy) * nglyphs);
         for (i = 0; i < nglyphs; i++) gidsCopy[i] = gids[i];
 
 	/* FIXME: Will this ever fail? */
@@ -386,7 +387,7 @@ static int dmxProcRenderAddGlyphs(ClientPtr client)
 		dmxSync(dmxScreen, FALSE);
 	    }
 	}
-        xfree(gidsCopy);
+        free(gidsCopy);
     }
 
     return ret;
@@ -410,7 +411,7 @@ static int dmxProcRenderFreeGlyphs(ClientPtr client)
 
 	nglyphs = ((client->req_len << 2) - sizeof(xRenderFreeGlyphsReq)) >> 2;
 	if (nglyphs) {
-            gids    = xalloc(sizeof(*gids) * nglyphs);
+            gids    = malloc(sizeof(*gids) * nglyphs);
             for (i = 0; i < nglyphs; i++)
                 gids[i] = ((CARD32 *)(stuff + 1))[i];
             
@@ -423,7 +424,7 @@ static int dmxProcRenderFreeGlyphs(ClientPtr client)
 		    dmxSync(dmxScreen, FALSE);
 		}
 	    }
-            xfree(gids);
+            free(gids);
 	}
     }
 
@@ -530,13 +531,13 @@ static int dmxProcRenderCompositeGlyphs(ClientPtr client)
 	/* The following only works for Render version > 0.2 */
 
 	/* All of the XGlyphElt* structure sizes are identical */
-	elts = xalloc(nelt * sizeof(XGlyphElt8));
+	elts = malloc(nelt * sizeof(XGlyphElt8));
 	if (!elts)
 	    return BadAlloc;
 
-	glyphs = xalloc(nglyph * size);
+	glyphs = malloc(nglyph * size);
 	if (!glyphs) {
-	    xfree(elts);
+	    free(elts);
 	    return BadAlloc;
 	}
 
@@ -604,8 +605,8 @@ static int dmxProcRenderCompositeGlyphs(ClientPtr client)
 
 	dmxSync(dmxScreen, FALSE);
 
-	xfree(elts);
-	xfree(glyphs);
+	free(elts);
+	free(glyphs);
     }
 
     return ret;
@@ -621,8 +622,7 @@ static int dmxProcRenderSetPictureTransform(ClientPtr client)
     REQUEST(xRenderSetPictureTransformReq);
 
     REQUEST_SIZE_MATCH(xRenderSetPictureTransformReq);
-    VERIFY_PICTURE(pPicture, stuff->picture, client, DixWriteAccess,
-		   RenderErrBase + BadPicture);
+    VERIFY_PICTURE(pPicture, stuff->picture, client, DixWriteAccess);
 
     /* For the following to work with PanoramiX, it assumes that Render
      * wraps the ProcRenderVector after dmxRenderInit has been called.
@@ -662,8 +662,7 @@ static int dmxProcRenderSetPictureFilter(ClientPtr client)
     REQUEST(xRenderSetPictureFilterReq);
 
     REQUEST_AT_LEAST_SIZE(xRenderSetPictureFilterReq);
-    VERIFY_PICTURE(pPicture, stuff->picture, client, DixWriteAccess,
-		   RenderErrBase + BadPicture);
+    VERIFY_PICTURE(pPicture, stuff->picture, client, DixWriteAccess);
 
     /* For the following to work with PanoramiX, it assumes that Render
      * wraps the ProcRenderVector after dmxRenderInit has been called.
@@ -872,14 +871,14 @@ int dmxChangePictureClip(PicturePtr pPicture, int clipType,
 				       pPictPriv->pict, 0, 0, None);
 	} else if (pPicture->clientClip) {
 	    RegionPtr   pClip = pPicture->clientClip;
-	    BoxPtr      pBox  = REGION_RECTS(pClip);
-	    int         nBox  = REGION_NUM_RECTS(pClip);
+	    BoxPtr      pBox  = RegionRects(pClip);
+	    int         nBox  = RegionNumRects(pClip);
 	    XRectangle *pRects;
 	    XRectangle *pRect;
 	    int         nRects;
 
 	    nRects = nBox;
-	    pRects = pRect = xalloc(nRects * sizeof(*pRect));
+	    pRects = pRect = malloc(nRects * sizeof(*pRect));
 
 	    while (nBox--) {
 		pRect->x      = pBox->x1;
@@ -895,7 +894,7 @@ int dmxChangePictureClip(PicturePtr pPicture, int clipType,
 					    0, 0,
 					    pRects,
 					    nRects);
-	    xfree(pRects);
+	    free(pRects);
 	} else {
 	    XRenderSetPictureClipRectangles(dmxScreen->beDisplay,
 					    pPictPriv->pict,
